#include <stdbool.h>
#include "SEGGER_RTT.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"
#include "app_util_platform.h"
#include "app_error.h"
//#include "boards.h"
#include <stdint.h>
#include "ADXL375_I2C.h" 

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
//#include "nrf_log_default_backends.h"

static const nrf_drv_twi_t * p_twi;

 ret_code_t ADXL375_I2C_register_write(uint8_t slave_addr,uint8_t * pdata, uint32_t bytes)
{
    /*The way to write to a register is in the following manner (I have skipped the ACKNOWLEDMENT, which is done for every byte transmitted):
       --> ((START condition: High-to-low transition while SCL is high (otherwise the bits are read while clock is low) ))
       --> Write address of sensor/slave on the SDA bus
       --> Set the SDA bus low for writing --> (0) (single bit/one clock cycle)
       --> Write the register address
       --> Write the data
       --> (STOP condition)
    */
    ret_code_t ret_code;
    ret_code = nrf_drv_twi_tx(p_twi, slave_addr, pdata, bytes , false);
    
    return ret_code;
}

 ret_code_t ADXL375_I2C_register_read(uint8_t slave_addr, uint8_t reg_addr,  uint8_t * pdata, uint32_t bytes)
{   
    /*The way to read a register is in the following manner (I have skipped the ACKNOWLEDMENT, which is done for every byte transmitted):
       --> Write address of sensor/slave on the SDA bus
       --> Set the SDA bus low for writing (0) (single bit/one clock cycle)
       --> Write the register address
       --> (STOP condition)
       --> (START condition)
       --> Write slave address
       --> Set the SDA bus high for reading --> (1) (single bit/one clock cycle)
       --> Now the slave will send the data from the register and master will listen and ACKNOWLEDGE
       -->
    */
    ret_code_t ret_code;
    
    ret_code = nrf_drv_twi_tx(p_twi,slave_addr, &reg_addr,1,false);
    
    if(ret_code != NRF_SUCCESS)
    {
        return ret_code;
    }
    
    ret_code = nrf_drv_twi_rx(p_twi,slave_addr, pdata, bytes);
    
    return ret_code;
}

 ret_code_t ADXL375_I2C_init(const nrf_drv_twi_t * p_twi_instance)
{
    ret_code_t ret_code;
    
    p_twi = p_twi_instance;

    uint8_t tx_data[2] = {POWER_CTL, 0x08}; 

    //The way i2c works is by sending one byte at the time, and the sensor will acknowlegdge for each byte
    //Here we are sending an array of 16 bits (2 bytes), the way to write to a register is to write the register address first
    //then write the data. Since the size is also sent, it knows how many bytes from that address it should write

    // Put the ADXL375 in Measurement Mode by writing 0x08 to the POWER_CTL Register
    ret_code = ADXL375_I2C_register_write(ADXL375_ADDR, tx_data, sizeof(tx_data) );
    
    return ret_code;
}

 void ADXL375_I2C_offsets_set(uint8_t x_offset, uint8_t y_offset,uint8_t z_offset)
{

    //  The value stored in the offset registers is automat-ically added to the acceleration data
    // However, there is actually three offset registers: OFSX, OFSY and OFSZ, I guess if you write exceed the first register
    // OFSX, it will automatically write to the next registers
    ret_code_t ret_code;
    
    uint8_t tx_data[4] = {OFSX, x_offset,y_offset,z_offset};
    
    ret_code = ADXL375_I2C_register_write(ADXL375_ADDR, tx_data, sizeof(tx_data));
    
    APP_ERROR_CHECK(ret_code);
    
    uint8_t rx_data[3] = {x_offset, y_offset, z_offset}; // {0, 0, 0};
    
    ret_code = ADXL375_I2C_register_read(ADXL375_ADDR, OFSX, rx_data, sizeof(rx_data)); 
    
    APP_ERROR_CHECK(ret_code);
    
    
    if( rx_data[0] == x_offset &&
        rx_data[1] == y_offset &&
        rx_data[2] == z_offset  )
    {
        NRF_LOG_INFO("Setting Offsets Sucessful! \n");
    }
    else
    {
        NRF_LOG_INFO("Setting Offsets Failed! \n");
    }
}

 void ADXL375_I2C_data_rate_set(uint8_t rate)
{
    ret_code_t ret_code;
    // The BW_RATE register (4 bits) controls the data rate
    // More info can be found here: https://www.analog.com/media/en/technical-documentation/data-sheets/ADXL375.PDF
    uint8_t tx_data[2] = {BW_RATE,rate};
    
    ret_code = ADXL375_I2C_register_write(ADXL375_ADDR,tx_data, sizeof(tx_data));
    
    APP_ERROR_CHECK(ret_code);
    
    uint8_t rx_data = 0x00;
    //This read is just for control (checking if write was successful)
    ret_code = ADXL375_I2C_register_read(ADXL375_ADDR, BW_RATE, &rx_data,1);
     
    APP_ERROR_CHECK(ret_code);
    
    if(rx_data == rate)
    {
        NRF_LOG_INFO("Setting Data rate Sucessful! \n");
    }
    else
    {
        NRF_LOG_INFO("Setting Data rate Failed! \n");
    }
}

void ADXL375_I2C_acc_data_read(int16_t * x_value, int16_t * y_value, int16_t * z_value)
{
    ret_code_t ret_code;
    
    uint8_t rx_buffer[6];
    
    // The DATAX0 register contains the acceleration data
    ret_code = ADXL375_I2C_register_read(ADXL375_ADDR, DATAX0, rx_buffer, sizeof(rx_buffer)); 
    
    APP_ERROR_CHECK(ret_code);
    
    // The reason it is done in this manner is because the buffer field 0 and 1, represents the x value
    // and you must left shift the second buffer, so it occupies the 8 MSB bits (same logic with y and z value)
    *x_value = (rx_buffer[1]<<8)|rx_buffer[0];
    *y_value = (rx_buffer[3]<<8)|rx_buffer[2];
    *z_value = (rx_buffer[5]<<8)|rx_buffer[4];
}


